<?php

namespace Drupal\oauth_server_sso\Form;

use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Symfony\Component\HttpFoundation\RedirectResponse;

class OauthUserConsent extends ConfirmFormBase {


  /**
   * User ID
   * @var integer
   */
  protected $user_id;

  /**
   * Destination url to redirect the user
   * @var string
   */
  protected $oauth_url;

  /**
   * {@inheritdoc}
   */
  public function getFormId() : string {
    return "oauth_user_consent_confirm";
  }

  public function buildForm(array $form, FormStateInterface $form_state, string $user = NULL)
  {
    $this->user_id = $user;
    $this->oauth_url = \Drupal::request()->query->get('oauth_url');
    return  parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    if($this->oauth_url !== NULL){
      global $base_url;
      $url = $base_url.'/'.urldecode($this->oauth_url);
      $response = new RedirectResponse($url);
      $response->send();
    }

  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {
    return new Url('entity.user.canonical', ['user' => $this->user_id]);
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    return $this->t('User OAuth Consent');
  }

  public function getDescription(){

    $url = urldecode($this->oauth_url);
    $queryString = parse_url($url, PHP_URL_QUERY);
    parse_str($queryString, $queryParams);

    $content = 'Below are the scopes requested by your client application. Granting these scopes will allow the client application to perform the following actions using OAuth.<br>';
    if (isset($queryParams['scope'])) {
      $requestedScopes = explode(' ', $queryParams['scope']);

      if (!empty($requestedScopes)) {
        $content .= '<ul>';
        foreach ($requestedScopes as $scope) {
          $content .= sprintf('<li><b>%s</b>: %s</li>', $scope, $this->getScopeDescription($scope));
        }
        $content .= '</ul>';
      }
    }

    return $this->t($content);
  }

  public function getConfirmText() {
    return $this->t('Allow');
  }

  public function getCancelText(){
    return $this->t('Deny');
  }

  private function getScopeDescription($scope_name): string {
    switch (strtolower($scope_name)){

      case 'profile':
        return 'This scope grants access to the users basic profile information';
        break;

      case 'openid':
        return 'This scope is necessary for authentication and authorization, enabling the client application to obtain an OpenID identity.';
        break;

      case 'email':
        return 'Granting this scope provides access to the users email address.';
        break;

      default:
        return '';

    }
  }

}